import UIKit
import VIDVLiveness

@objc(Vidvliveness)
class Vidvliveness: NSObject {
    
    let dispatchGroup = DispatchGroup()
    var jsonResponse = ""
    var success = false
    var pluginInitialzingError = "6001";
    var capturedActionsArray: [VIDVDetectedFace] = []
    
    @objc(startLiveness:withResolver:withRejecter:)
    func startLiveness(_ params: NSDictionary,_ resolve: @escaping RCTPromiseResolveBlock,_ reject: @escaping RCTPromiseRejectBlock) {
        
        guard let baseURL = params.value(forKey: "base_url") as? String else {
            reject(pluginInitialzingError, "please make sure you entered base url correctly", .none);
            return
        }
        guard let accessToken = params.value(forKey: "access_token") as? String else {
            reject(pluginInitialzingError, "please make sure you entered access token correctly", .none);
            return
        }
        guard let bundle = params.value(forKey: "bundle_key") as? String else {
            reject(pluginInitialzingError, "please make sure you entered bundle key correctly", .none);
            return
        }
        
        let enableSmile  = params.value(forKey: "enable_smile") as? Bool ?? true
        let enableLeft  = params.value(forKey: "enable_look_left") as? Bool ?? true
        let enableRight  = params.value(forKey: "enable_look_right") as? Bool ?? true
        let enableEyes  = params.value(forKey: "enable_close_eyes") as? Bool ?? true
        
        let languageString = params.value(forKey: "language") as? String ?? ""
        let trials  = params.value(forKey: "livenss_number_of_failed_trials") as? Int
        let timer  = params.value(forKey: "liveness_time_per_action") as? Int
        let instructions  = params.value(forKey: "livness_number_of_instructions") as? Int
        
        let facematchImage  = params.value(forKey: "facematch_image") as? String
        let facematchOcrTransactionId  = params.value(forKey: "facematch_ocr_transactionId") as? String
        let enableVoiceover = params.value(forKey: "enable_voiceover") as? Bool ?? true
        let showErrorMessage = params.value(forKey: "show_error_message") as? Bool ?? true
        let headers  = params.value(forKey: "headers") as? [String:String] ?? [:]
        let primaryColor  = params.value(forKey: "primary_color") as? String ?? ""
        
        
        // creds
        var settings = VIDVLivenessBuilder()
            .setBundleKey(bundle)
            .setAccessToken(accessToken)
            .setBaseURL(baseURL)
        
        // liveness actions
        if !enableEyes {
            settings = settings.withoutCloseEyes()
        }
        if !enableLeft {
            settings = settings.withoutLookLeft()
        }
        if !enableRight {
            settings = settings.withoutLookRight()
        }
        if !enableSmile {
            settings = settings.withoutSmile()
        }
        
        // liveness settings
        if languageString != "" {
            settings = settings.setLanguage(languageString)
        }
        if let trials = trials {
            settings = settings.setFailTrials(trials)
        }
        if let timer = timer {
            settings = settings.setInstructionTimer(timer)
        }
        if let instructionsCount = instructions {
            settings = settings.setNumberOfInstructions(instructionsCount)
        }
        
        // face match
        if let facematchOcrTransactionId = facematchOcrTransactionId  {
            settings = settings.setFrontTransactionID(facematchOcrTransactionId)
        } else if let facematchImage = facematchImage {
            let data = Data(base64Encoded: facematchImage ) ?? Data()
            let image = UIImage(data: data)
            settings = settings.setFaceMatchImage(image)
        }
        
        // generic settings
        if !enableVoiceover {
            settings = settings.withoutVoiceOver()
        }
        if primaryColor != "" {
            let customColor = hexStringToUIColor(hex: primaryColor)
            settings = settings.setPrimaryColor(customColor)
        }
        if !headers.isEmpty {
            settings = settings.setHeaders(headers)
        }
        settings = settings.showErrorDialogs(showErrorMessage)
        
        DispatchQueue.main.async {
            if #available(iOS 11.0, *) {
                
                let keyWindow = UIApplication.shared.windows.filter {$0.isKeyWindow}.first
                guard var topController = keyWindow?.rootViewController else {return}
                while let presentedViewController = topController.presentedViewController {
                    topController = presentedViewController
                }
                settings.start(vc: topController, livenessDelegate: self)
            } else {
                // Fallback on earlier versions (we don't support earlier versions)
            }
        }
        self.dispatchGroup.enter()
        dispatchGroup.notify(queue: DispatchQueue.global()){
            if self.success {
                resolve(self.jsonResponse)
            } else {
                resolve(self.jsonResponse)
            }
        }
    }
    
    func hexStringToUIColor(hex: String?) -> UIColor {
        guard let hex = hex  else {return hexStringToUIColor(hex: "#62CBC9")}
        var cString: String = hex.trimmingCharacters(in: .whitespacesAndNewlines).uppercased()
        
        if cString.hasPrefix("#") {
            cString.remove(at: cString.startIndex)
        }
        
        if (cString.count) != 6 {
            return UIColor.gray
        }
        
        var rgbValue: UInt64 = 0
        Scanner(string: cString).scanHexInt64(&rgbValue)
        
        return UIColor(
            red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
            green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
            blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
            alpha: CGFloat(1.0)
        )
    }
}

extension Vidvliveness : VIDVLivenessDelegate {
    func onLivenessResult(_ VIDVLivenessResponse: VIDVLiveness.VIDVLivenessResponse) {
        let dict = NSMutableDictionary()
        let nameValuePairs = NSMutableDictionary()
        dict["nameValuePairs"] = nameValuePairs
        
        switch VIDVLivenessResponse {
        case .success(VIDVLivenessResult: let result):
            let VIDVReslut = encodeLivenessResults(VIDVResult: result)
            nameValuePairs["state"] = "SUCCESS"
            nameValuePairs["livenessResult"] = VIDVReslut
            nameValuePairs["capturedImages"] = self.getEncodedCapturedImages()
            success = true
            returnData(dict: dict)
        case .serviceFailure(VIDVLivenessResult: let result, errorCode: let errorCode, errorMessage: let errorMessage):
            let VIDVReslut = encodeLivenessResults(VIDVResult: result)
            nameValuePairs["state"] = "FAILURE"
            nameValuePairs["errorCode"] = errorCode
            nameValuePairs["errorMessage"] = errorMessage
            nameValuePairs["livenessResult"] = VIDVReslut
            nameValuePairs["capturedImages"] = self.getEncodedCapturedImages()
            success = false
            returnData(dict: dict)
        case .builderError(errorCode: let errorCode, errorMessage: let errorMessage):
            nameValuePairs["state"] = "ERROR"
            nameValuePairs["errorCode"] = errorCode
            nameValuePairs["errorMessage"] = errorMessage
            success = false
            returnData(dict: dict)
        case .userExited(VIDVLivenessResult: let result, step: let step):
            let VIDVReslut =  encodeLivenessResults(VIDVResult: result)
            nameValuePairs["state"] = "EXIT"
            nameValuePairs["step"] = step
            nameValuePairs["livenessResult"] = VIDVReslut
            nameValuePairs["capturedImages"] = self.getEncodedCapturedImages()
            success = false
            returnData(dict: dict)
        case .capturedActions(capturedActions: let capturedActions):
            capturedActionsArray.append(capturedActions)
        @unknown default:
            break;
        }
    }
    
    func encodeLivenessResults(VIDVResult: VIDVLivenessResult?) -> [String: Any] {
        if let VIDVResult = VIDVResult {
            do {
                let jsonEncoder = JSONEncoder()
                let jsonData = try jsonEncoder.encode(VIDVResult)
                let json = try JSONSerialization.jsonObject(with: jsonData, options: []) as? [String : Any]
                return json ?? [:]
            } catch {
                print("Failed to encode VIDV Result: \(error.localizedDescription)")
                return [:]
            }
        } else {
            return [:]
        }
    }
    
    func getEncodedCapturedImages() -> [String: Any] {
        do {
            let jsonEncoder = JSONEncoder()
            let jsonData = try jsonEncoder.encode(self.capturedActionsArray)
            let json = try JSONSerialization.jsonObject(with: jsonData, options: []) as? [String : Any]
            return json ?? [:]
        } catch {
            print("Failed to encode VIDV Result: \(error.localizedDescription)")
        }
        return [:]
    }
    
    func returnData(dict: NSMutableDictionary){
        do {
            let jsonData = try JSONSerialization.data(withJSONObject: dict, options: .prettyPrinted)
            let json = String(data: jsonData, encoding: .utf8)
            self.jsonResponse = json ?? ""
            dispatchGroup.leave()
        } catch let error as NSError {
            print("Failed to load: \(error.localizedDescription)")
        }
    }
}